//*****************************************************************************
// FILE..........: BTreeArrayConstIterator.hpp
// DESCRIPTION...: const iterator for BTreeArray
// AUTHOR........: Dmitry Vinogradov
// COPYRIGHT.....: Copyright(c) FastCpp 2012-2014, Dmitry Vinogradov
// LICENSE.......: "2 EUR" or "advertising note at the about box"
//                 See http://www.FastCpp.com/license.html
//
// HISTORY.......: DATE       COMMENT
//                 ---------- ------------------------------------------------
//                     y  m  d
//                 2013-02-25 Created by VDima.
//                 2013-04-14 Renamed BTreeArrayIterator.hpp -> BTreeArrayConstIterator.hpp
//*****************************************************************************

//---------------------------------------------------------------- include guard
#ifndef FastCpp_BTREEARRAYCONSTITERATOR_HPP
#define FastCpp_BTREEARRAYCONSTITERATOR_HPP

//------------------------------------------------------------------------------
#ifndef              FastCpp_BTREEARRAYITERATOR_HPP
#include <FastCpp/BTreeArray/BTreeArrayIterator.hpp>
#endif//             FastCpp_BTREEARRAYITERATOR_HPP
//------------------------------------------------------------------------------

namespace FastCpp
{

//-------------------------------------------------BTreeArrayConstIterator class
//| |       |                 |                 |                            |
template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
class BTreeArrayConstIterator : public std::iterator<std::bidirectional_iterator_tag, T, ptrdiff_t, const T*, const T&>
{
  private:
    typedef Detail::ElementReference<T, UIndex, BIndex, BLOCK_SIZE, Allocator> ElementReferenceType;
    typedef BTreeArrayIterator      <T, UIndex, BIndex, BLOCK_SIZE, Allocator> BTreeArrayIteratorType;
  public:
    typedef typename std::iterator<
                                   std::bidirectional_iterator_tag,
                                   T,
                                   ptrdiff_t,
                                   const T*,
                                   const T&
                                  >::difference_type DifferenceType;
  public:
    inline                    BTreeArrayConstIterator();
    inline                    BTreeArrayConstIterator
                                                (const ElementReferenceType& rRef);
    inline                    BTreeArrayConstIterator
                                                (const BTreeArrayIteratorType& rIterator);
  public:
    inline  BTreeArrayConstIterator&
                              operator =        (const BTreeArrayConstIterator& rObj);
    inline  BTreeArrayConstIterator&
                              operator =        (const BTreeArrayIteratorType& rIterator);
  public:
    inline  UIndex            GetIndex          () const;
    inline  BTreeArrayIteratorType
                              Insert            (const T& rElemt);
    inline  BTreeArrayIteratorType
                              Erase             ();
  public:
    inline  const T&          operator *        () const;
    inline  const T*          operator ->       () const;
    inline  const T&          operator []       (DifferenceType nOffset) const;

    inline  bool              operator ==       (const BTreeArrayConstIterator& rObj) const;
    inline  bool              operator !=       (const BTreeArrayConstIterator& rObj) const;
    inline  BTreeArrayConstIterator&
                              operator ++       ();    // prefix
    inline  BTreeArrayConstIterator
                              operator ++       (int); // postfix
    inline  BTreeArrayConstIterator&
                              operator --       ();    // prefix
    inline  BTreeArrayConstIterator
                              operator --       (int); // postfix
    inline  BTreeArrayConstIterator&
                              operator +=       (DifferenceType nOffset);
    inline  BTreeArrayConstIterator
                              operator +        (DifferenceType nOffset) const;
    inline  BTreeArrayConstIterator&
                              operator -=       (DifferenceType nOffset);
    inline  BTreeArrayConstIterator
                              operator -        (DifferenceType nOffset) const;
    inline  DifferenceType    operator -        (const BTreeArrayConstIterator& rConstIterator) const;

  private:
    inline  bool              Eq                (const BTreeArrayConstIterator& rObj) const;
  private:
            BTreeArrayIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>
                              m_oIterator;
};


//--------------------------------------------------------------- public methods


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::BTreeArrayConstIterator()
  : m_oIterator()
{
}


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::BTreeArrayConstIterator(const ElementReferenceType& rRef)
  : m_oIterator(rRef)
{
}


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::BTreeArrayConstIterator
(
  const BTreeArrayIteratorType& rIterator
)
  : m_oIterator(rIterator)
{
}


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>&
BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::operator = (const BTreeArrayConstIterator& rObj)
{
  m_oIterator = rObj.m_oIterator;
  return *this;
}


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>&
BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::operator = (const BTreeArrayIteratorType& rIterator)
{
  m_oIterator = rIterator;
  return *this;
}


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline UIndex BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::GetIndex() const
{
  return m_oIterator.GetIndex();
}


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline typename BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::BTreeArrayIteratorType
BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::Insert(const T& rElemt)
{
  return m_oIterator.Insert(rElemt);
}


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline typename BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::BTreeArrayIteratorType
BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::Erase()
{
  return m_oIterator.Erase();
}


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline const T& BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::operator * () const
{
  return *m_oIterator;
}


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline const T* BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::operator -> () const
{
  return m_oIterator.operator->();
}


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline const T& BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::operator [] (DifferenceType nOffset) const
{
  return m_oIterator[nOffset];
}


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline bool BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::operator == (const BTreeArrayConstIterator& rObj) const
{
  return m_oIterator == rObj.m_oIterator;
}


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline bool BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::operator != (const BTreeArrayConstIterator& rObj) const
{
  return m_oIterator != rObj.m_oIterator;
}


// ++ prefix
template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>&
BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::operator ++ ()
{
  ++m_oIterator;
  return *this;
}


// postfix ++
template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>
BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::operator ++ (int) 
{
  BTreeArrayConstIterator oRet(*this);
  ++m_oIterator;
  return oRet;
}


// -- prefix
template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>&
BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::operator -- ()
{
  --m_oIterator;
  return *this;
}


// postfix
template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>
BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::operator -- (int)
{
  BTreeArrayConstIterator oRet(*this);
  --m_oIterator;
  return oRet;
}


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>&
BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::operator += (DifferenceType nOffset)
{
  m_oIterator += nOffset;
  return *this;
}


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>
BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::operator + (DifferenceType nOffset) const
{
  BTreeArrayConstIterator oRet(*this);
  return oRet += nOffset;
}


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>&
BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::operator -= (DifferenceType nOffset)
{
  return *this += -nOffset;
}                              


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>
BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::operator - (DifferenceType nOffset) const
{
  BTreeArrayConstIterator oRet(*this);
  return oRet -= nOffset;
}


template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline typename BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::DifferenceType
BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::operator - (const BTreeArrayConstIterator& rConstIterator) const
{
  return GetIndex() - rConstIterator.GetIndex();
}


//------------------------------------------------------- private inline methods

template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
inline bool BTreeArrayConstIterator<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::Eq(const BTreeArrayConstIterator& rObj) const
{
  return m_oIterator == rObj.m_oIterator;
}


} // namespace FastCpp
#endif//FastCpp_BTREEARRAYCONSTITERATOR_HPP
//** (END OF FILE  : BTreeArrayConstIterator.hpp) ******************************
