//*****************************************************************************
// FILE..........: BTreeArrayBlockInterface.hpp
// AUTHOR........: Dmitry Vinogradov
// COPYRIGHT.....: Copyright(c) FastCpp 2012-2014, Dmitry Vinogradov
// LICENSE.......: "2 EUR" or "advertising note at the about box"
//                 See http://www.FastCpp.com/license.html
//
// HISTORY.......: DATE       COMMENT
//                 ---------- --------------------------------------------------
//                     y  m  d
//                 2013-06-21 Created by Dmitry Vinogradov
//******************************************************************************

//---------------------------------------------------------------- include guard
#ifndef FastCpp_BTREEARRAYBLOCKINTERFACE_HPP
#define FastCpp_BTREEARRAYBLOCKINTERFACE_HPP

//--------------------------------------------------------------- import std lib
//------------------------------------------------------------------------------
namespace FastCpp
{
namespace Detail
{
template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator> class BTreeArrayUpBlockInterface;
template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator> class ElementReference;

//------------------------------------------------------------------------------
//| |       |                 |                 |                            |
template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
class BTreeArrayBlockInterface
{
  public:
    typedef BTreeArrayUpBlockInterface<T, UIndex, BIndex, BLOCK_SIZE, Allocator> BTreeArrayUpBlockInterfaceType;
    typedef ElementReference          <T, UIndex, BIndex, BLOCK_SIZE, Allocator> ElementReferenceType;
  public:
    virtual BTreeArrayBlockInterface*
                              Clone             (Allocator& rAllocator) const = 0;
    virtual void              Deallocate        (Allocator& rAllocator) = 0;
    virtual void              DestroyAndDeallocate
                                                (Allocator& rAllocator) = 0;

    virtual                  ~BTreeArrayBlockInterface()
                              {
                              }
    virtual void              SetUpNode         (BTreeArrayUpBlockInterfaceType* pUpNode) = 0;
    virtual BTreeArrayBlockInterface*
                              Insert            (
                                                  UIndex                    nUIndex,
                                                  const T&                  rElemt,
                                                  BTreeArrayBlockInterface* pArrPrev,
                                                  BTreeArrayBlockInterface* pNext,
                                                  Allocator&                rAllocator
                                                ) = 0;
    virtual T                 Replace           (UIndex nUIndex, const T& rBy) = 0;
    virtual UIndex            GetCount          () = 0;
    virtual BIndex            GetItemsCount     () const = 0;
    virtual UIndex            Depth             () const = 0;
    virtual BTreeArrayBlockInterface*
                              PushFront         (const T& rElemt, Allocator& rAllocator) = 0;
    virtual BTreeArrayBlockInterface*
                              Add               (const T& rElemt, Allocator& rAllocator) = 0;
    virtual BTreeArrayBlockInterface*
                              PlusOne           (Allocator& rAllocator) = 0;
    virtual T&                Get               (UIndex nUIndex) = 0;
    virtual T&                GetFirst          () = 0;
    virtual T&                GetLast           () = 0;
    virtual void              Constate          () = 0;
    virtual void              ShrinkToFit       (Allocator& rAllocator) = 0;
    virtual UIndex            Delete            (
                                                  UIndex     nUIndex,
                                                  T&         rRet,
                                                  bool&      bSubBlockPacked,
                                                  Allocator& rAllocator
                                                ) = 0;
    virtual bool              DeleteRange       (
                                                  UIndex     nFirst,
                                                  UIndex     nLast,
                                                  Allocator& rAllocator
                                                ) = 0;
    virtual void              RemoveLast        (T& rRet, Allocator& rAllocator) = 0;
    virtual UIndex            CutTailFromIndex  (
                                                  UIndex     nUIndex,
                                                  Allocator& rAllocator
                                                ) = 0;
    virtual bool              UniteWith         (
                                                  BTreeArrayBlockInterface* pRight,
                                                  Allocator&                rAllocator
                                                ) = 0;
    virtual BTreeArrayBlockInterface*
                              RemoveUpper       () = 0;
    virtual ElementReferenceType
                              Begin             () = 0;
    virtual ElementReferenceType
                              Last              () = 0;
    virtual ElementReferenceType
                              GetElementReference
                                                (UIndex nUIndex) = 0;
    virtual BTreeArrayBlockInterface*
                              GetFirstBlockOnLevel
                                                (UIndex nULevel);
    virtual BTreeArrayBlockInterface*
                              GetLastBlockOnLevel
                                                (UIndex nULevel);
};

//------------------------------------------------------------------------------
//------------------------------------------------- inline function declarations
//--------------------------------------------------------------- public methods

//virtual
template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
BTreeArrayBlockInterface<T, UIndex, BIndex, BLOCK_SIZE, Allocator>*
BTreeArrayBlockInterface<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::GetFirstBlockOnLevel(UIndex nULevel)
{
  assert(0 == nULevel);
  return NULL;
}


//virtual
template <class T, class UIndex, class BIndex, BIndex BLOCK_SIZE, class Allocator>
BTreeArrayBlockInterface<T, UIndex, BIndex, BLOCK_SIZE, Allocator>*
BTreeArrayBlockInterface<T, UIndex, BIndex, BLOCK_SIZE, Allocator>::GetLastBlockOnLevel(UIndex nULevel)
{
  assert(0 == nULevel);
  return this;
}

//------------------------------------------------------------ protected methods
//-------------------------------------------------------------- private methods
//------------------------------------------------------------- inline functions
}// namespace Detail
}// namespace FastCpp
#endif//FastCpp_BTREEARRAYBLOCKINTERFACE_HPP
//** (END OF FILE  : BTreeArrayBlockInterface.hpp) *****************************
