//******************************************************************************
// FILE..........: std_vector.hpp
// DESCRIPTION...: "std::vector" based on BTreeArray
// AUTHOR........: Dmitry Vinogradov
// COPYRIGHT.....: Copyright(c) FastCpp 2014, Dmitry Vinogradov
// LICENSE.......: "2 EUR" or "advertising note at the about box"
//                 See http://www.FastCpp.com/license.html
//
// HISTORY.......: DATE       COMMENT
//                 ---------- ------------------------------------------------
//                     y  m  d
//                 2013-03-28 Created by VDima.
//******************************************************************************

//---------------------------------------------------------------- include guard
#ifndef FastCpp_STD_VECTOR_HPP
#define FastCpp_STD_VECTOR_HPP

//--------------------------------------------------------------- import std lib
#include <algorithm>
#include <memory>

//------------------------------------------------------------------------------
#ifndef              FastCpp_BTREEARRAY_HPP
#include <FastCpp/BTreeArray/BTreeArray.hpp>
#endif//             FastCpp_BTREEARRAY_HPP
//-----------------------------------------------------------------------------
                                                             
namespace FastCpp
{
//-----------------------------------------------------------------------------
//| |       |                 |                 |                            |

template <class T, class Allocator = std::allocator<T> >
class std_vector : protected BTreeArray<T, size_t, unsigned char, 8, 8, Allocator>
{
  public:
    // special types:
    typedef       BTreeArray<T, size_t, unsigned char, 8, 8, Allocator> container_type;
    typedef       container_type&                                       container_reference;
    typedef const container_type&                                       container_const_reference;
  public:
    // types:
    typedef          T                                               value_type;
    typedef          value_type&                                     reference;
    typedef          const value_type&                               const_reference;
    typedef typename container_type::IteratorType                    iterator;
    typedef typename container_type::ConstIteratorType               const_iterator;
    typedef typename container_type::SizeType                        size_type;
    typedef typename Allocator::difference_type                      difference_type;
    typedef          Allocator                                       allocator_type;
    typedef typename Allocator::pointer                              pointer;
    typedef typename Allocator::const_pointer                        const_pointer;
    typedef typename container_type::ReverseIteratorType             reverse_iterator;
    typedef typename container_type::ConstReverseIteratorType        const_reverse_iterator;
  public:
    inline
    explicit                  std_vector        (const Allocator& = Allocator());
    inline
    explicit                  std_vector        (size_type n);
    inline                    std_vector        (
                                                  size_type        n,
                                                  const T&         value,
                                                  const Allocator& = Allocator()
                                                );
    inline                    std_vector        (const std_vector<T, Allocator>& rObj);
    inline                    std_vector        (
                                                  const std_vector& oVector,
                                                  const Allocator&  rAllocator
                                                );
    template <class InputIterator>
    inline                    std_vector        (
                                                  InputIterator first,
                                                  InputIterator last,
                                                  const Allocator& = Allocator()
                                                );
    virtual                  ~std_vector        ();
  public:
    inline  container_reference
                              GetContainer      ();
    inline  container_const_reference
                              GetContainer      () const;
  public:
    inline  std_vector&       operator =        (const std_vector& rObj);
    template <class InputIterator>
    inline  void              assign            (InputIterator first, InputIterator last);
    inline  void              assign            (size_type n, const T& rValue);

    template <class InputIterator>
    inline  void              assign            (
                                                  InputIterator       first,
                                                  InputIterator       last,
                                                  NotIntegralType_Tag selector
                                                );
    template <class SomeIntegralType>
    inline  void              assign            (
                                                  SomeIntegralType nCount,
                                                  SomeIntegralType nValue,
                                                  IntegralType_Tag selector
                                                );
    inline  allocator_type    get_allocator() const;// noexcept;
    // iterators:
    inline  iterator          begin             ();
    inline  const_iterator    begin             () const;
    inline  iterator          end               ();
    inline  const_iterator    end               () const;
    inline  reverse_iterator  rbegin            ();
    inline  const_reverse_iterator
                              rbegin            () const;
    inline  reverse_iterator  rend              ();
    inline  const_reverse_iterator
                              rend              () const;
    inline  const_iterator    cbegin            () const;
    inline  const_iterator    cend              () const;
    inline  const_reverse_iterator
                              crbegin           () const;
    inline  const_reverse_iterator
                              crend             () const;

    inline  size_type         size              () const;
    inline  size_type         max_size          () const;
    inline  void              resize            (size_type sz);
    inline  void              resize            (size_type sz, const T& c);
//size_type capacity() const noexcept;
    inline  bool              empty             () const;
//void reserve(size_type n);
    inline  void              shrink_to_fit     ();
    // element access:
    inline  reference         operator []       (size_type n);
    inline  const_reference   operator []       (size_type n) const;
    inline  reference         at                (size_type n);
    inline  const_reference   at                (size_type n) const;
    inline  reference         front             ();
    inline  const_reference   front             () const;
    inline  reference         back              ();
    inline  const_reference   back              () const;
                              //todo
                              //T* data() noexcept;
                              //const T* data() const noexcept;
    inline  void              push_back         (const T& x);
    inline  void              pop_back          ();
    inline  iterator          insert            (const_iterator position, const T& x);
    inline  iterator          insert            (
                                                  const_iterator position,
                                                  size_type      n,
                                                  const T&       x
                                                );
    template <class InputIterator>
    inline  iterator          insert            (
                                                  const_iterator position,
                                                  InputIterator  first,
                                                  InputIterator  last
                                                );
    inline  iterator          erase             (const_iterator position);
    inline  iterator          erase             (
                                                  const_iterator first,
                                                  const_iterator last
                                                );
    inline  void              swap              (std_vector<T, Allocator>& v);
    inline  void              clear             ();
};

//--------------------------------------------[ PUBLIC FUNCTION DECLARATIONS ]=

template <class T, class Allocator>
inline bool operator == (const std_vector<T, Allocator>& x, const std_vector<T, Allocator>& y);

template <class T, class Allocator>
inline bool operator < (const std_vector<T, Allocator>& x, const std_vector<T, Allocator>& y);

template <class T, class Allocator>
inline bool operator != (const std_vector<T, Allocator>& x, const std_vector<T, Allocator>& y);

template <class T, class Allocator>
inline bool operator > (const std_vector<T, Allocator>& x, const std_vector<T, Allocator>& y);

template <class T, class Allocator>
inline bool operator >= (const std_vector<T, Allocator>& x, const std_vector<T, Allocator>& y);

template <class T, class Allocator>
inline bool operator <= (const std_vector<T, Allocator>& x, const std_vector<T, Allocator>& y);

template <class T, class Allocator>
inline void swap(std_vector<T, Allocator>& x, std_vector<T, Allocator>& y);

//--------------------------------------------------------------- public methods

template <class T, class Allocator>
inline std_vector<T, Allocator>::std_vector(const Allocator& rAllocator)
  : container_type(rAllocator)
{
}


template <class T, class Allocator>
inline std_vector<T, Allocator>::std_vector(size_type nSize)
  : container_type(nSize)
{
}


template <class T, class Allocator>
inline std_vector<T, Allocator>::std_vector
(
  size_type        nSize, 
  const T&         rValue,
  const Allocator& rAllocator
)
  : container_type(nSize, rValue, rAllocator)
{
}


template <class T, class Allocator>
template <class InputIterator>
inline std_vector<T, Allocator>::std_vector
(
  InputIterator    first,
  InputIterator    last,
  const Allocator& rAllocator
)
  : container_type(rAllocator)
{
  // fill with [_First, _Last)
  typename IsIntegral<InputIterator>::Type selector;
  assign(first, last, selector);
}


template <class T, class Allocator>
inline std_vector<T, Allocator>::std_vector(const std_vector<T, Allocator>& rObj)
  : container_type(rObj)
{
}


template <class T, class Allocator>
inline std_vector<T, Allocator>::std_vector
(
  const std_vector& oVector,
  const Allocator&  rAllocator
)
  : container_type(oVector, rAllocator)
{
}


//virtual
template <class T, class Allocator>
std_vector<T, Allocator>::~std_vector()
{
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::container_reference
std_vector<T, Allocator>::GetContainer()
{
    return *this;
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::container_const_reference
std_vector<T, Allocator>::GetContainer() const
{
    return *this;
}


template <class T, class Allocator>
inline std_vector<T, Allocator>& std_vector<T, Allocator>::operator = (const std_vector& rObj)
{
  this->Assign(rObj);
  return *this;
}


template <class T, class Allocator>
template <class InputIterator>
inline void std_vector<T, Allocator>::assign(InputIterator first, InputIterator last)
{
  typename IsIntegral<InputIterator>::Type selector;
  assign(first, last, selector);
}


template <class T, class Allocator>
inline void std_vector<T, Allocator>::assign(size_type n, const T& rValue)
{
  this->Assign(n, rValue);
}


template <class T, class Allocator>
template <class InputIterator>
inline void std_vector<T, Allocator>::assign
(
  InputIterator       first,
  InputIterator       last,
  NotIntegralType_Tag //selector
)
{
  this->AssignIts(first, last);
}

    
template <class T, class Allocator>
template <class SomeIntegralType>
inline void std_vector<T, Allocator>::assign
(
  SomeIntegralType nCount,
  SomeIntegralType nValue,
  IntegralType_Tag //selector
)
{
  this->Assign(nCount, nValue);
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::allocator_type
std_vector<T, Allocator>::get_allocator() const
{
  return this->GetAllocator();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::iterator
std_vector<T, Allocator>::begin()
{
  return this->Begin();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::const_iterator
std_vector<T, Allocator>::begin() const
{
  return this->Begin();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::iterator
std_vector<T, Allocator>::end()
{
  return this->End();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::const_iterator
std_vector<T, Allocator>::end() const
{
  return this->End();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::reverse_iterator
std_vector<T, Allocator>::rbegin()
{
  return this->RBegin();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::const_reverse_iterator
std_vector<T, Allocator>::rbegin() const
{
  return this->RBegin();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::reverse_iterator
std_vector<T, Allocator>::rend()
{
  return this->REnd();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::const_reverse_iterator
std_vector<T, Allocator>::rend() const
{
  return this->REnd();
}

           
template <class T, class Allocator>
inline typename std_vector<T, Allocator>::const_iterator
std_vector<T, Allocator>::cbegin() const
{
  return this->ConstBegin();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::const_iterator
std_vector<T, Allocator>::cend() const
{
  return this->ConstEnd();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::const_reverse_iterator
std_vector<T, Allocator>::crbegin() const
{
  return this->ConstRBegin();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::const_reverse_iterator
std_vector<T, Allocator>::crend() const
{
  return this->ConstREnd();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::size_type
std_vector<T, Allocator>::size() const
{
  return this->container_type::GetCount();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::size_type
std_vector<T, Allocator>::max_size() const
{
  return this->container_type::GetMaxSize();
}


template <class T, class Allocator>
inline void std_vector<T, Allocator>::resize(size_type sz)
{
  this->Resize(sz);
}


template <class T, class Allocator>
inline void std_vector<T, Allocator>::resize(size_type sz, const T& rValue)
{
  this->Resize(sz, rValue);
}


template <class T, class Allocator>
inline bool std_vector<T, Allocator>::empty() const
{
  return this->IsEmpty();
}


template <class T, class Allocator>
inline void std_vector<T, Allocator>::shrink_to_fit()
{
  return this->ShrinkToFit();
}

            
template <class T, class Allocator>
inline typename std_vector<T, Allocator>::reference
std_vector<T, Allocator>::operator [] (size_type n)
{
  return this->Get(n);
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::const_reference
std_vector<T, Allocator>::operator [] (size_type n) const
{
  return this->Get(n);
}

            
template <class T, class Allocator>
inline typename std_vector<T, Allocator>::reference
std_vector<T, Allocator>::at(size_type n)
{
  return this->At(n);
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::const_reference
std_vector<T, Allocator>::at(size_type n) const
{
  return this->At(n);
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::reference
std_vector<T, Allocator>::front()
{
  return this->GetFirst();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::const_reference
std_vector<T, Allocator>::front() const
{
  return this->GetFirst();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::reference
std_vector<T, Allocator>::back()
{
  return this->GetLast();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::const_reference
std_vector<T, Allocator>::back() const
{
  return this->GetLast();
}


template <class T, class Allocator>
inline void std_vector<T, Allocator>::push_back(const T& x)
{
  this->PushBack(x);
}


template <class T, class Allocator>
inline void std_vector<T, Allocator>::pop_back()
{
  this->RemoveLast();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::iterator
std_vector<T, Allocator>::insert(const_iterator position, const T& x)
{
  return position.Insert(x);
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::iterator
std_vector<T, Allocator>::insert
(
  const_iterator position,
  size_type      n,
  const T&       x
)
{
  return this->InsertIt(position, n, x);
}


template <class T, class Allocator>
template <class InputIterator>
inline typename std_vector<T, Allocator>::iterator
std_vector<T, Allocator>::insert
(
  const_iterator position,
  InputIterator  first,
  InputIterator  last
)
{
  return this->container_type::insert(position, first, last);
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::iterator
std_vector<T, Allocator>::erase(const_iterator position)
{
  return position.Erase();
}


template <class T, class Allocator>
inline typename std_vector<T, Allocator>::iterator
std_vector<T, Allocator>::erase
(
  const_iterator first,
  const_iterator last
)
{
  return this->RemoveIts(first, last);
}


template <class T, class Allocator>
inline void std_vector<T, Allocator>::swap(std_vector<T, Allocator>& v)
{
  this->Swap(v);
}


template <class T, class Allocator>
inline void std_vector<T, Allocator>::clear()
{
  this->Clear();
}

//------------------------------------------------------ public inline functions

template <class T, class Allocator>
inline bool operator == (const std_vector<T, Allocator>& x, const std_vector<T, Allocator>& y)
{
  if ( x.size() != y.size() )
    return false;

#ifdef _MSC_VER
// suppress C4996: 'std::equal': Function call with parameters that may be
// unsafe - this call relies on the caller to check that the passed values
// are correct.
#pragma warning( push )
#pragma warning( disable : 4996 )
#endif//_MSC_VER

  return std::equal(x.begin(), x.end(), y.begin());

#ifdef _MSC_VER
#pragma warning( pop ) 
#endif//_MSC_VER
}


template <class T, class Allocator>
inline bool operator != (const std_vector<T, Allocator>& x, const std_vector<T, Allocator>& y)
{
  return !(x == y);
}


template <class T, class Allocator>
inline bool operator < (const std_vector<T, Allocator>& x, const std_vector<T, Allocator>& y)
{
  return std::lexicographical_compare(x.begin(), x.end(), y.begin(), y.end());
}


template <class T, class Allocator>
inline bool operator > (const std_vector<T, Allocator>& x, const std_vector<T, Allocator>& y)
{
  return y < x;
}


template <class T, class Allocator>
inline bool operator >= (const std_vector<T, Allocator>& x, const std_vector<T, Allocator>& y)
{
  return !(x < y);
}


template <class T, class Allocator>
inline bool operator <= (const std_vector<T, Allocator>& x, const std_vector<T, Allocator>& y)
{
  return !(y < x);
}


template <class T, class Allocator>
inline void swap(std_vector<T, Allocator>& x, std_vector<T, Allocator>& y)
{
  x.swap(y);
}


}// namespace FastCpp
#endif//FastCpp_STD_VECTOR_HPP
//** (END OF FILE  : std_vector.hpp) ******************************************
