//*****************************************************************************
// FILE..........: SIntPow.hpp
// AUTHOR........: Dmitry Vinogradov
// COPYRIGHT.....: Copyright(c) FastCpp 2012-2013, Dmitry Vinogradov
// LICENSE.......: "2 EUR" or "advertising note at the about box"
//                 See http://www.FastCpp.com/license.html
//
// HISTORY.......: DATE       COMMENT
//                 ---------- --------------------------------------------------
//                     y  m  d
//                 2013-10-12 Created by Dmitry Vinogradov
//******************************************************************************

//---------------------------------------------------------------- include guard
#ifndef FastCpp_SINTPOW_HPP
#define FastCpp_SINTPOW_HPP

//--------------------------------------------------------------- import std lib
//------------------------------------------------------------------------------
#include <FastCpp/Metafunctions/UIntPow.hpp>
//------------------------------------------------------------------------------
//| |       |                 |                 |                            |
namespace FastCpp
{


//                                    N times
//                          /---------^---------\
// SIntPow<a, N>::result == (a * a * a * ... * a)
//
// Example 
//  assert((FastCpp::SIntPow<2,  3>::result == 8));
//
// SIntPow<N, M>::result equals "N raised to the power of M"
//                       equals 0 if integral overflow occures
// 
template<signed int NValue, unsigned int NPower>
struct SIntPow {
    static const signed int result = INT_MAX < UIntPow<NValue, NPower>::result
                                       ? 0
                                       : UIntPow<NValue, NPower>::result;
};


template<signed int NValue>
struct SIntPow<NValue, 1U> {
    static const signed int result = NValue;
};


template<signed int NValue>
struct SIntPow<NValue, 0U> {
    static const signed int result = 1;
};


// SIntPowEx<N, M, D>::result equals "N raised to the power of M"
//                            equals D if integral overflow occures
// 
template<unsigned int NValue, unsigned int NPower, unsigned int NOverflowValue>
struct SIntPowEx {
    static const unsigned int result = SIntPow<NValue, NPower>::result <= 0
                                         ? NOverflowValue
                                         : SIntPow<NValue, NPower>::result;
};


}// namespace FastCpp
//------------------------------------------------------------------------------
//------------------------------------------------- inline function declarations
//--------------------------------------------------------------- public methods
//------------------------------------------------------------ protected methods
//-------------------------------------------------------------- private methods
//------------------------------------------------------------- inline functions
#endif//FastCpp_SINTPOW_HPP

//** (END OF FILE  : SIntPow.hpp) **********************************************
