//*****************************************************************************
// FILE..........: SLongPow.hpp
// AUTHOR........: Dmitry Vinogradov
// COPYRIGHT.....: Copyright(c) FastCpp 2012-2013, Dmitry Vinogradov
// LICENSE.......: "2 EUR" or "advertising note at the about box"
//                 See http://www.FastCpp.com/license.html
//
// HISTORY.......: DATE       COMMENT
//                 ---------- --------------------------------------------------
//                     y  m  d
//                 2013-10-12 Created by Dmitry Vinogradov
//******************************************************************************

//---------------------------------------------------------------- include guard
#ifndef FastCpp_SLONGPOW_HPP
#define FastCpp_SLONGPOW_HPP

//--------------------------------------------------------------- import std lib
#include <limits.h>
//------------------------------------------------------------------------------
#include <FastCpp/Metafunctions/ULongPow.hpp>

namespace FastCpp
{


//                                     N times
//                           /---------^---------\
// SLongPow<a, N>::result == (a * a * a * ... * a)
//
// Example 
//  assert((FastCpp::SLongPow<2,  3>::result == 8));
//
// SLongPow<N, M>::result equals "N raised to the power of M"
//                        equals 0 if integral overflow occures
// 
template<signed long NValue, unsigned int NPower>
struct SLongPow {
    static const signed long result = LONG_MAX < ULongPow<NValue, NPower>::result
                                       ? 0
                                       : ULongPow<NValue, NPower>::result;
};


template<signed long NValue>
struct SLongPow<NValue, 1U> {
    static const signed long result = NValue;
};


template<signed long NValue>
struct SLongPow<NValue, 0U> {
    static const signed long result = 1;
};


// SLongPowEx<N, M, D>::result equals "N raised to the power of M"
//                             equals D if integral overflow occures
// 
template<unsigned int NValue, unsigned int NPower, unsigned int NOverflowValue>
struct SLongPowEx {
    static const unsigned int result = SLongPow<NValue, NPower>::result <= 0
                                         ? NOverflowValue
                                         : SLongPow<NValue, NPower>::result;
};

//------------------------------------------------------------------------------
}// namespace FastCpp
#endif//FastCpp_SLONGPOW_HPP
//** (END OF FILE  : SLongPow.hpp) *********************************************
