//*****************************************************************************
// FILE..........: ULongPow.hpp
// AUTHOR........: Dmitry Vinogradov
// COPYRIGHT.....: Copyright(c) FastCpp 2012-2013, Dmitry Vinogradov
// LICENSE.......: "2 EUR" or "advertising note at the about box"
//                 See http://www.FastCpp.com/license.html
//
// HISTORY.......: DATE       COMMENT
//                 ---------- --------------------------------------------------
//                     y  m  d
//                 2013-10-12 Created by Dmitry Vinogradov
//******************************************************************************

//---------------------------------------------------------------- include guard
#ifndef FastCpp_ULONGPOW_HPP
#define FastCpp_ULONGPOW_HPP

//--------------------------------------------------------------- import std lib
#include <limits.h>
//------------------------------------------------------------------------------
namespace FastCpp
{


//                                     N times
//                           /---------^---------\
// ULongPow<a, N>::result == (a * a * a * ... * a)
//
// Example 
//  assert((FastCpp::ULongPow<2,  3>::result == 8));
//
// ULongPow<N, M>::result equals "N raised to the power of M"
//                        equals 0 if integral overflow occures
// 
template<unsigned long NValue, unsigned int NPower>
struct ULongPow {
    static const unsigned long result = ULONG_MAX / NValue <= ULongPow<NValue, NPower - 1>::result
                                         ? 0
                                         : NValue * ULongPow<NValue, NPower - 1>::result;
};


template<unsigned long NValue>
struct ULongPow<NValue, 1U> {
    static const unsigned long result = NValue;
};


template<unsigned long NValue>
struct ULongPow<NValue, 0U> {
    static const unsigned long result = 1;
};


// ULongPowEx<N, M, D>::result equals "N raised to the power of M"
//                             equals D if integral overflow occures
// 
template<unsigned int NValue, unsigned int NPower, unsigned int NOverflowValue>
struct ULongPowEx {
    static const unsigned int result = ULongPow<NValue, NPower>::result <= 0
                                         ? NOverflowValue
                                         : ULongPow<NValue, NPower>::result;
};

//------------------------------------------------------------------------------
}// namespace FastCpp
#endif//FastCpp_ULONGPOW_HPP
//** (END OF FILE  : ULongPow.hpp) *********************************************
