//******************************************************************************
// FILE..........: SpeedTest.cpp
// DESCRIPTION...: speed test
// AUTHOR........: Dmitry Vinogradov
// COPYRIGHT.....: Copyright(c) FastCpp 2014, Dmitry Vinogradov
// LICENSE.......: "2 EUR" or "advertising note at the about box"
//                 See http://www.FastCpp.com/license.html
//
// HISTORY.......: DATE       COMMENT
//                 ---------- -------------------------------------------------
//                     y  m  d
//                 2014-09-19 Created by Dmitry Vinogradov
//                 2014-09-21 Renamed BTreeArrayTest_Speed_insert0.cpp
//                                    ->
//                                    SpeedTest.cpp
//******************************************************************************

#ifdef _MSC_VER
#ifdef NDEBUG
#define STRING2(x) #x
#define STRING(x) STRING2(x)
// http://msdn.microsoft.com/en-us/library/aa985896.aspx
#pragma message (__FILE__ " _SECURE_SCL is " STRING(_SECURE_SCL))
#pragma message (__FILE__ " _ITERATOR_DEBUG_LEVEL is " STRING(_ITERATOR_DEBUG_LEVEL))
#endif
#endif

#include <assert.h>
#include <deque>
#include <iostream>
#include <stdio.h>
#include <time.h>
#include <vector>


//-------------------------------------------------------------------------------------------------------------
// if in Visual Studio you have the error:
// fatal error C1083: Cannot open include file: 'FastCpp/BTreeArray/BTreeArray.hpp': No such file or directory
// in the following line:
#include <FastCpp/BTreeArray/BTreeArray.hpp>
// then the FastCpp enviroment variable does not have correct path.
// See FastCpp\SetEnvFastCpp.bat file.
//-------------------------------------------------------------------------------------------------------------


#ifdef _MSC_VER
//------------------------------------------------------------------------------
#include <Windows.h>

#define Ticks    DWORD
#define TicksNow GetTickCount

inline long DiffMilliseconds(const DWORD nFinish, const DWORD nStart)
{
  return nFinish - nStart;
}
//------------------------------------------------------------------------------
#else
//------------------------------------------------------------------------------
#include <sys/time.h>
#define Ticks  timeval 

inline timeval TicksNow()
{
  timeval stNow;
  stNow.tv_sec  = 0;
  stNow.tv_usec = 0;
  gettimeofday(&stNow, NULL);

  return stNow;
}


inline long DiffMilliseconds(timeval stFinish, timeval stStart)
{
  timeval stDiff;
  timersub(&stFinish, &stStart, &stDiff);

  long nMilliseconds = stDiff.tv_usec / 1000;

  assert(0 <= nMilliseconds);

  if ( 500 < stDiff.tv_usec - nMilliseconds * 1000 )
    nMilliseconds += 1;

  nMilliseconds += stDiff.tv_sec * 1000;

  return nMilliseconds;
}
//------------------------------------------------------------------------------
#endif


//--------------------------------------------------------------------destructor


template <class TContainer>
long Container_destructor(const int nCount)
{
  std::cout << "destructor of " << nCount << " items started...";

  Ticks tStart;
  {
    TContainer  v;

    for(int i = 0; i < nCount; i++)
      v.push_back(i);

    tStart = TicksNow();
  }

  const Ticks tEnd  = TicksNow();
  const long  nDiff = DiffMilliseconds(tEnd, tStart);
  std::cout << " finished\n";
  return nDiff;
}


//-----------------------------------------------------------------push_front(i)


template <class TContainer>
long Container_push_front(const int nCount)
{
  std::cout << "push_front(" << nCount << ") started...";
  const Ticks tStart = TicksNow();
  TContainer  v;

  for(int i = 0; i < nCount; i++)
    v.push_front(i);

  const Ticks tEnd  = TicksNow();
  const long  nDiff = DiffMilliseconds(tEnd, tStart);
  std::cout << " finished\n";
  return nDiff;
}


//------------------------------------------------------------------push_back(i)


template <class TContainer>
long Container_push_back(const int nCount)
{
  std::cout << "push_back(" << nCount << ") started...";

  const Ticks tStart = TicksNow();
  TContainer  v;

  for(int i = 0; i < nCount; i++)
    v.push_back(i);

  const Ticks tEnd  = TicksNow();
  const long  nDiff = DiffMilliseconds(tEnd, tStart);
  std::cout << " finished\n";
  return nDiff;
}


//----------------------------------------------------------insert(v.begin(), i)


template <class TContainer>
long Container_insert_begin(const int nCount)
{
  std::cout << "insert_begin(" << nCount << ") started...";
  const Ticks tStart = TicksNow();
  TContainer  v;

  for(int i = 0; i < nCount; i++)
    v.insert(v.begin(), i);

  const Ticks tEnd  = TicksNow();
  const long  nDiff = DiffMilliseconds(tEnd, tStart);
  std::cout << " finished\n";

  return nDiff;
}



//--------------------------------------------------insert(v.begin() + i / 2, i)


template <class TContainer>
long Container_InsertIdiv2(const int nCount)
{
  std::cout << "InsertIdiv2(" << nCount << ") started...";
  const Ticks tStart = TicksNow();
  TContainer  v;

  for(int i = 0; i < nCount; i++)
    v.insert(v.begin() + i / 2, i);

  const Ticks tEnd  = TicksNow();
  const long  nDiff = DiffMilliseconds(tEnd, tStart);
  std::cout << " finished\n";
  return nDiff;
}


//-------------------------------------------------------------------operator []


template <class TContainer>
long Container_IndexAccess(const int nCount)
{
  std::cout << "Container[i](" << nCount << ") started...";

  TContainer  v;

  for(int i = 0; i < nCount; i++)
    v.push_back(i);

  const Ticks tStart = TicksNow();

  for(int i = 0; i < nCount; i++)
    v[i] = i + 1;

  const Ticks tEnd  = TicksNow();
  const long  nDiff = DiffMilliseconds(tEnd, tStart);
  std::cout << " finished\n";
  return nDiff;
}


//---------------------------------------------------------------operator [i+=2]


template <class TContainer>
long Container_IndexAccess2(const int nCount)
{
  std::cout << "Container[i+=2](" << nCount << ") started...";

  TContainer  v;

  for(int i = 0; i < nCount; i++)
    v.push_back(i);

  const Ticks tStart = TicksNow();

  for(int i = 0; i < nCount; i += 2)
    v[i] = i + 1;

  const Ticks tEnd  = TicksNow();
  const long  nDiff = DiffMilliseconds(tEnd, tStart);
  std::cout << " finished\n";
  return nDiff;
}


//-------------------------------------------------------------------it 0 -> end


template <class TContainer>
long Container_IteratorTraverse(const int nCount)
{
  std::cout << "it = begin()-->end() (" << nCount << ") started...";

  TContainer v;

  for(int i = 0; i < nCount; i++)
    v.push_back(i);

  const Ticks tStart = TicksNow();

  int i = 0;
  for(typename TContainer::iterator it = v.begin(), itEnd = v.end(); it != itEnd; ++it)
    *it = i++;

  const Ticks tEnd  = TicksNow();
  const long  nDiff = DiffMilliseconds(tEnd, tStart);
  std::cout << " finished\n";
  return nDiff;
}


//------------------------------------------------------------it 0 -> end step 2


template <class TContainer>
long Container_IteratorTraverse2(const int nCount)
{
  std::cout << "it = begin()-->end() step 2 (" << nCount << ") started...";

  TContainer  v;

  for(int i = 0; i < nCount; i++)
    v.push_back(i);

  if ( nCount % 2 == 1 )  // must be even for step 2
    v.push_back(nCount);

  const Ticks tStart = TicksNow();

  int i = 0;
  for(typename TContainer::iterator it = v.begin(), itEnd = v.end(); it != itEnd; it += 2)
    *it = i++;

  const Ticks tEnd  = TicksNow();
  const long  nDiff = DiffMilliseconds(tEnd, tStart);
  std::cout << " finished\n";
  return nDiff;
}


//--------------------------------------------------------------erase(v.begin())


template <class TContainer>
long Container_erase_begin(const int nCount)
{
  std::cout << "erase(v.begin()) (" << nCount << ") started...";

  TContainer  v;

  for(int i = 0; i < nCount; i++)
    v.push_back(i);

  const Ticks tStart = TicksNow();

  while(0 < v.size())
    v.erase(v.begin());

  const Ticks tEnd  = TicksNow();
  const long  nDiff = DiffMilliseconds(tEnd, tStart);
  std::cout << " finished\n";
  return nDiff;
}

                                                                
//----------------------------------------------------------------------pop_back


template <class TContainer>
long Container_pop_back(const int nCount)
{
  std::cout << "pop_back (" << nCount << ") started...";

  TContainer  v;

  for(int i = 0; i < nCount; i++)
    v.push_back(i);

  const Ticks tStart = TicksNow();

  while(0 < v.size())
    v.pop_back();

  const Ticks tEnd  = TicksNow();
  const long  nDiff = DiffMilliseconds(tEnd, tStart);
  std::cout << " finished\n";
  return nDiff;
}


//-------------------------------------------------erase(v.begin() + v.size()/2)


template <class TContainer>
long Container_erase_middle(const int nCount)
{
  std::cout << "erase middle (" << nCount << ") started...";

  TContainer  v;

  for(int i = 0; i < nCount; i++)
    v.push_back(i);

  const Ticks tStart = TicksNow();

  while(0 < v.size())
    v.erase(v.begin() + v.size() / 2);

  const Ticks tEnd  = TicksNow();
  const long  nDiff = DiffMilliseconds(tEnd, tStart);
  std::cout << " finished\n";
  return nDiff;
}

//------------------------------------------------------------------------------


int main(int argc, char* argv[])
{
  std::cout << "SpeedTest for std::vector<int>         \n"
               "              std::deque<int>          \n" 
               "              FastCpp::BTreeArray<int> \n"
               "version: 1.0193\n"
  "Portions of this software use the FastCpp,\n"
  "Copyright (c) 2012-2014, Dmitry Vinogradov, www.FastCpp.com\n"
  "\n"
  "The full test can take 3 hours,\n"
  "the first iteration takes approximately 25 minutes\n\n";

  time_t tTestBeginning = time(NULL);
  std::cout << "start: " << asctime(localtime(&tTestBeginning)) << std::endl;

  namespace F = FastCpp;

//  const int    nItemsCount =   500;
//  const int    nBigCount   = 100000;
  const int    nItemsCount =   500000;
  const int    nBigCount   = 10000000;
  const size_t nCycles     = 8;
  const size_t nMethods    = 12;
  const size_t nContainers = 3;

  long arrMilliseconds[nCycles][nMethods][nContainers];
  int  arrOpCount[nMethods];

  int m = 0;// method
  for(size_t i = 0; i < nCycles; i++)
  {
    m = 0;
    arrOpCount        [m]     = nBigCount;
    arrMilliseconds[i][m][0]  = Container_destructor<       std::vector<int>   >(arrOpCount[m]); printf("Destructor of std::vector   (%d items) took %ld ms\n\n",       arrOpCount[m], arrMilliseconds[i][m][0]);
    arrMilliseconds[i][m][1]  = Container_destructor<       std::deque<int>    >(arrOpCount[m]); printf("Destructor of std::deque    (%d items) took %ld ms\n\n",       arrOpCount[m], arrMilliseconds[i][m][1]);
    arrMilliseconds[i][m][2]  = Container_destructor<       F::BTreeArray<int> >(arrOpCount[m]); printf("Destructor of F::BTreeArray (%d items) took %ld ms\n\n",       arrOpCount[m], arrMilliseconds[i][m][2]);

    m++;
    arrOpCount        [m]     = nBigCount;
    arrMilliseconds[i][m][0]  = 0;//Container_push_front<   std::vector<int>   >(arrOpCount[m]);                                                                         
    arrMilliseconds[i][m][1]  = Container_push_front<       std::deque<int>    >(arrOpCount[m]); printf("%d of std::deque::   push_front(i) took %ld ms\n\n",           arrOpCount[m], arrMilliseconds[i][0][1]);
    arrMilliseconds[i][m][2]  = Container_push_front<       F::BTreeArray<int> >(arrOpCount[m]); printf("%d of F::BTreeArray::push_front(i) took %ld ms\n\n",           arrOpCount[m], arrMilliseconds[i][0][2]);
                             
    m++;
    arrOpCount        [m]     = nBigCount;                                                                                                                                            
    arrMilliseconds[i][m][0]  = Container_push_back<        std::vector<int>   >(arrOpCount[m]); printf("%d of std::vector::  push_back(i) took %ld ms\n\n",            arrOpCount[m], arrMilliseconds[i][m][0]);
    arrMilliseconds[i][m][1]  = Container_push_back<        std::deque<int>    >(arrOpCount[m]); printf("%d of std::deque::   push_back(i) took %ld ms\n\n",            arrOpCount[m], arrMilliseconds[i][m][1]);
    arrMilliseconds[i][m][2]  = Container_push_back<        F::BTreeArray<int> >(arrOpCount[m]); printf("%d of F::BTreeArray::push_back(i) took %ld ms\n\n",            arrOpCount[m], arrMilliseconds[i][m][2]);
                            
    m++;
    arrOpCount        [m]     = nItemsCount;
    arrMilliseconds[i][m][0]  = Container_insert_begin<     std::vector<int>   >(arrOpCount[m]); printf("%d of std::vector::  insert(.begin(), i) took %ld ms\n\n",     arrOpCount[m], arrMilliseconds[i][m][0]);
    arrMilliseconds[i][m][1]  = Container_insert_begin<     std::deque<int>    >(arrOpCount[m]); printf("%d of std::deque::   insert(.begin(), i) took %ld ms\n\n",     arrOpCount[m], arrMilliseconds[i][m][1]);
    arrMilliseconds[i][m][2]  = Container_insert_begin<     F::BTreeArray<int> >(arrOpCount[m]); printf("%d of F::BTreeArray::insert(.begin(), i) took %ld ms\n\n",     arrOpCount[m], arrMilliseconds[i][m][2]);
                            
    m++;
    arrOpCount        [m]     = nItemsCount;
    arrMilliseconds[i][m][0]  = Container_InsertIdiv2<      std::vector<int>   >(arrOpCount[m]); printf("%d of std::vector::  insert(.begin()+i/2, i) took %ld ms\n\n", arrOpCount[m], arrMilliseconds[i][m][0]);
    arrMilliseconds[i][m][1]  = Container_InsertIdiv2<      std::deque<int>    >(arrOpCount[m]); printf("%d of std::deque::   insert(.begin()+i/2, i) took %ld ms\n\n", arrOpCount[m], arrMilliseconds[i][m][1]);
    arrMilliseconds[i][m][2]  = Container_InsertIdiv2<      F::BTreeArray<int> >(arrOpCount[m]); printf("%d of F::BTreeArray::insert(.begin()+i/2, i) took %ld ms\n\n", arrOpCount[m], arrMilliseconds[i][m][2]);
                            
    m++;
    arrOpCount        [m]     = nBigCount;
    arrMilliseconds[i][m][0]  = Container_IndexAccess<      std::vector<int>   >(arrOpCount[m]); printf("%d of std::vector::  operator[i] took %ld ms\n\n",             arrOpCount[m], arrMilliseconds[i][m][0]);
    arrMilliseconds[i][m][1]  = Container_IndexAccess<      std::deque<int>    >(arrOpCount[m]); printf("%d of std::deque::   operator[i] took %ld ms\n\n",             arrOpCount[m], arrMilliseconds[i][m][1]);
    arrMilliseconds[i][m][2]  = Container_IndexAccess<      F::BTreeArray<int> >(arrOpCount[m]); printf("%d of F::BTreeArray::operator[i] took %ld ms\n\n",             arrOpCount[m], arrMilliseconds[i][m][2]);
                            
    m++;
    arrOpCount        [m]     = nBigCount;
    arrMilliseconds[i][m][0]  = Container_IndexAccess2<     std::vector<int>   >(arrOpCount[m]); printf("%d of std::vector::  operator[i+=2] took %ld ms\n\n",          arrOpCount[m], arrMilliseconds[i][m][0]);
    arrMilliseconds[i][m][1]  = Container_IndexAccess2<     std::deque<int>    >(arrOpCount[m]); printf("%d of std::deque::   operator[i+=2] took %ld ms\n\n",          arrOpCount[m], arrMilliseconds[i][m][1]);
    arrMilliseconds[i][m][2]  = Container_IndexAccess2<     F::BTreeArray<int> >(arrOpCount[m]); printf("%d of F::BTreeArray::operator[i+=2] took %ld ms\n\n",          arrOpCount[m], arrMilliseconds[i][m][2]);
                            
    m++;
    arrOpCount        [m]     = nBigCount;
    arrMilliseconds[i][m][0]  = Container_IteratorTraverse< std::vector<int>   >(arrOpCount[m]); printf("%d of std::vector::  it:begin()-->end() took %ld ms\n\n",      arrOpCount[m], arrMilliseconds[i][m][0]);
    arrMilliseconds[i][m][1]  = Container_IteratorTraverse< std::deque<int>    >(arrOpCount[m]); printf("%d of std::deque::   it:begin()-->end() took %ld ms\n\n",      arrOpCount[m], arrMilliseconds[i][m][1]);
    arrMilliseconds[i][m][2]  = Container_IteratorTraverse< F::BTreeArray<int> >(arrOpCount[m]); printf("%d of F::BTreeArray::it:begin()-->end() took %ld ms\n\n",      arrOpCount[m], arrMilliseconds[i][m][2]);
                            
    m++;
    arrOpCount        [m]     = nBigCount;
    arrMilliseconds[i][m][0]  = Container_IteratorTraverse2<std::vector<int>   >(arrOpCount[m]); printf("%d of std::vector::  it:begin()-->end()+=2 took %ld ms\n\n",   arrOpCount[m], arrMilliseconds[i][m][0]);
    arrMilliseconds[i][m][1]  = Container_IteratorTraverse2<std::deque<int>    >(arrOpCount[m]); printf("%d of std::deque::   it:begin()-->end()+=2 took %ld ms\n\n",   arrOpCount[m], arrMilliseconds[i][m][1]);
    arrMilliseconds[i][m][2]  = Container_IteratorTraverse2<F::BTreeArray<int> >(arrOpCount[m]); printf("%d of F::BTreeArray::it:begin()-->end()+=2 took %ld ms\n\n",   arrOpCount[m], arrMilliseconds[i][m][2]);
                            
    m++;
    arrOpCount        [m]     = nItemsCount;
    arrMilliseconds[i][m][0]  = Container_erase_begin<      std::vector<int>   >(arrOpCount[m]); printf("%d of std::vector::  erase(.begin()) took %ld ms\n\n",         arrOpCount[m], arrMilliseconds[i][m][0]);
    arrMilliseconds[i][m][1]  = Container_erase_begin<      std::deque<int>    >(arrOpCount[m]); printf("%d of std::deque::   erase(.begin()) took %ld ms\n\n",         arrOpCount[m], arrMilliseconds[i][m][1]);
    arrMilliseconds[i][m][2]  = Container_erase_begin<      F::BTreeArray<int> >(arrOpCount[m]); printf("%d of F::BTreeArray::erase(.begin()) took %ld ms\n\n",         arrOpCount[m], arrMilliseconds[i][m][2]);
                            
    m++;
    arrOpCount        [m]     = nBigCount;
    arrMilliseconds[i][m][0]  = Container_pop_back<         std::vector<int>   >(arrOpCount[m]); printf("%d of std::vector::  pop_back() took %ld ms\n\n",              arrOpCount[m], arrMilliseconds[i][m][0]);
    arrMilliseconds[i][m][1]  = Container_pop_back<         std::deque<int>    >(arrOpCount[m]); printf("%d of std::deque::   pop_back() took %ld ms\n\n",              arrOpCount[m], arrMilliseconds[i][m][1]);
    arrMilliseconds[i][m][2]  = Container_pop_back<         F::BTreeArray<int> >(arrOpCount[m]); printf("%d of F::BTreeArray::pop_back() took %ld ms\n\n",              arrOpCount[m], arrMilliseconds[i][m][2]);

    m++;
    arrOpCount        [m]    = nItemsCount;
    arrMilliseconds[i][m][0] = Container_erase_middle<      std::vector<int>   >(arrOpCount[m]); printf("%d of std::vector::  erase(.begin()+i/2) took %ld ms\n\n",     arrOpCount[m], arrMilliseconds[i][m][0]);
    arrMilliseconds[i][m][1] = Container_erase_middle<      std::deque<int>    >(arrOpCount[m]); printf("%d of std::deque::   erase(.begin()+i/2) took %ld ms\n\n",     arrOpCount[m], arrMilliseconds[i][m][1]);
    arrMilliseconds[i][m][2] = Container_erase_middle<      F::BTreeArray<int> >(arrOpCount[m]); printf("%d of F::BTreeArray::erase(.begin()+i/2) took %ld ms\n\n",     arrOpCount[m], arrMilliseconds[i][m][2]);
    

    printf( "%d of %d\n"
            "-------------------------------------------------------------------------------\n"
            "|                        | std::     | std::     | FastCpp::     |  operation |\n"
            "|                        |    vector |     deque |    BTreeArray |    count   |\n"
            "|------------------------|-----------|-----------|---------------|------------|\n"
            "|destructor              | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
            "|push_front(i)           | " "   %6c | " "  %7ld | " "      %7ld |"    " %10d |\n"
            "|push_back(i)            | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
            "|insert(v.begin(), i)    | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
            "|insert(v.begin()+i/2, i)| " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
            "|operator []             | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
            "|operator [] step 2      | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
            "|it : begin()-->end()    | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
            "|it : begin()-->end() +=2| " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
            "|erase(v.begin())        | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
            "|pop_back()              | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
            "|erase middle            | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
            "-------------------------------------------------------------------------------\n",
            static_cast<int>(i + 1), static_cast<int>(nCycles), 
            arrMilliseconds[i][ 0][0], arrMilliseconds[i][ 0][1], arrMilliseconds[i][ 0][2], arrOpCount[ 0],
                                  '-', arrMilliseconds[i][ 1][1], arrMilliseconds[i][ 1][2], arrOpCount[ 1],
            arrMilliseconds[i][ 2][0], arrMilliseconds[i][ 2][1], arrMilliseconds[i][ 2][2], arrOpCount[ 2],
            arrMilliseconds[i][ 3][0], arrMilliseconds[i][ 3][1], arrMilliseconds[i][ 3][2], arrOpCount[ 3],
            arrMilliseconds[i][ 4][0], arrMilliseconds[i][ 4][1], arrMilliseconds[i][ 4][2], arrOpCount[ 4],
            arrMilliseconds[i][ 5][0], arrMilliseconds[i][ 5][1], arrMilliseconds[i][ 5][2], arrOpCount[ 5],
            arrMilliseconds[i][ 6][0], arrMilliseconds[i][ 6][1], arrMilliseconds[i][ 6][2], arrOpCount[ 6],
            arrMilliseconds[i][ 7][0], arrMilliseconds[i][ 7][1], arrMilliseconds[i][ 7][2], arrOpCount[ 7],
            arrMilliseconds[i][ 8][0], arrMilliseconds[i][ 8][1], arrMilliseconds[i][ 8][2], arrOpCount[ 8],
            arrMilliseconds[i][ 9][0], arrMilliseconds[i][ 9][1], arrMilliseconds[i][ 9][2], arrOpCount[ 9],
            arrMilliseconds[i][10][0], arrMilliseconds[i][10][1], arrMilliseconds[i][10][2], arrOpCount[10],
            arrMilliseconds[i][11][0], arrMilliseconds[i][11][1], arrMilliseconds[i][11][2], arrOpCount[11]
          );
  }

  long arrAverages[nMethods][nContainers];

  for(size_t m = 0; m < nMethods; m++)
  {
    for(size_t c = 0; c < nContainers; c++)
      arrAverages[m][c] = 0;
  }

  for(size_t m = 0; m < nMethods; m++)
  {
    for(size_t c = 0; c < nContainers; c++)
    {
      long nSum = 0;
      for(size_t i = 0; i < nCycles; i++)
        nSum += arrMilliseconds[i][m][c];

      arrAverages[m][c] = nSum / nCycles;
    }
  }

  printf( "\n\nAverages (milliseconds)\n"
          "-------------------------------------------------------------------------------\n"
          "|                        | std::     | std::     | FastCpp::     |  operation |\n"
          "|                        |    vector |     deque |    BTreeArray |    count   |\n"
          "|------------------------|-----------|-----------|---------------|------------|\n"
          "|destructor              | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
          "|push_front(i)           | " "   %6c | " "  %7ld | " "      %7ld |"    " %10d |\n"
          "|push_back(i)            | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
          "|insert(v.begin(), i)    | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
          "|insert(v.begin()+i/2, i)| " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
          "|operator []             | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
          "|operator [] step 2      | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
          "|it : begin()-->end()    | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
          "|it : begin()-->end() +=2| " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
          "|erase(v.begin())        | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
          "|pop_back()              | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
          "|erase middle            | " "  %7ld | " "  %7ld | " "      %7ld |"    " %10d |\n"
          "-------------------------------------------------------------------------------\n",
          arrAverages[ 0][0], arrAverages[ 0][1], arrAverages[ 0][2], arrOpCount[ 0],
                         '-', arrAverages[ 1][1], arrAverages[ 1][2], arrOpCount[ 1],
          arrAverages[ 2][0], arrAverages[ 2][1], arrAverages[ 2][2], arrOpCount[ 2],
          arrAverages[ 3][0], arrAverages[ 3][1], arrAverages[ 3][2], arrOpCount[ 3],
          arrAverages[ 4][0], arrAverages[ 4][1], arrAverages[ 4][2], arrOpCount[ 4],
          arrAverages[ 5][0], arrAverages[ 5][1], arrAverages[ 5][2], arrOpCount[ 5],
          arrAverages[ 6][0], arrAverages[ 6][1], arrAverages[ 6][2], arrOpCount[ 6],
          arrAverages[ 7][0], arrAverages[ 7][1], arrAverages[ 7][2], arrOpCount[ 7],
          arrAverages[ 8][0], arrAverages[ 8][1], arrAverages[ 8][2], arrOpCount[ 8],
          arrAverages[ 9][0], arrAverages[ 9][1], arrAverages[ 9][2], arrOpCount[ 9],
          arrAverages[10][0], arrAverages[10][1], arrAverages[10][2], arrOpCount[10],
          arrAverages[11][0], arrAverages[11][1], arrAverages[11][2], arrOpCount[11]
        );

  time_t tTestEnd = time(NULL);
  std::cout << "finish: " << asctime(localtime(&tTestEnd)) << std::endl;
  double fSeconds = difftime(tTestEnd, tTestBeginning);
  printf("%.f seconds elapsed\n", fSeconds);

	return 0;
}

  

//** (END OF FILE  : SpeedTest.hpp) ********************************************

